@@ -68,7 +68,7 @@ Buffer *CBuf;
 Line *CLine;
 Buffer_Local_Type Buffer_Local = /*{{{*/
 {
-   8, 0, 0
+   8, 0, 0, 72
 };
 
 /*}}}*/
@@ -352,7 +352,9 @@ int jed_multibyte_charcasecmp (unsigned char **ap, unsigned char *amax,
    return -1;
 }
 
-/* The buffer is required to be at least JED_MAX_MULTIBYTE_SIZE bytes wide */
+/* The buffer is required to be at least JED_MAX_MULTIBYTE_SIZE bytes wide.
+ * This routine does not \0 terminate the buffer.
+ */
 unsigned char *jed_wchar_to_multibyte (SLwchar_Type c, unsigned char *buf)
 {
 #if JED_HAS_UTF8_SUPPORT
@@ -435,26 +437,42 @@ void jed_count_chars (void) /*{{{*/
 	  sprintf (buf, "-<%0X>, ", (int) w);
         else
           {
-             SLuchar_Type *p, *pmax;
-             
+             SLuchar_Type *p, *p1, *pmax;
+             int skip_combining = 0;
+
              p = CLine->data + Point;
              if (*p == 0)
                {
                   p = (SLuchar_Type *) "^@";
                   pmax = p + 2;
+		  skip_combining = 1;
                }
              else
                pmax = jed_multibyte_chars_forward (p, CLine->data + CLine->len, 1, NULL, 1);
              
              buf[0] = '\'';
              strncpy (buf + 1, (char *)p, pmax - p);
-             p = (SLuchar_Type *) (buf + 1 + (pmax - p));
+             p1 = (SLuchar_Type *) (buf + 1 + (pmax - p));
 
-             sprintf ((char *)p, "'=%ld/0x%lX/%#lo, ", (long)w, (unsigned long)w, (unsigned long)w);
+             sprintf ((char *)p1, "'=%ld/0x%lX/%#lo", (long)w, (unsigned long)w, (unsigned long)w);
+	     /* Check for combining */
+	     if ((skip_combining == 0) && Jed_UTF8_Mode)
+	       {
+		  unsigned int dn;
+		  unsigned int count = 1;
+		  p = SLutf8_decode (p, pmax, &w, &dn);   /* skip first */
+		  while ((p != NULL) && (p < pmax) && (count < SLSMG_MAX_CHARS_PER_CELL))
+		    {
+		       p = SLutf8_decode (p, pmax, &w, &dn);
+		       if (p != NULL)
+			 sprintf (buf + strlen(buf), " + 0x%lX", (long)w);
+		       count++;
+		    }
+	       }
           }
      }
    
-   sprintf (buf + strlen (buf), "point %lu of %lu", m, n);
+   sprintf (buf + strlen (buf), ", point %lu of %lu", m, n);
    SLang_push_string(buf);
 }
 
@@ -938,7 +956,9 @@ unsigned char *remake_line(unsigned int size) /*{{{*/
      {
 #if 1				       /* NOTE: This was #if 0.  Why??? */
 #ifndef KEEP_SPACE_INFO
-	if (((CLine->len + 3) & mask) == size) return d;
+	if ((((CLine->len + 3) & mask) == size)
+	    && (size + 8 < (unsigned) CLine->len))/* realloc if the new size is less than the old */
+	  return d;
 #endif
 #endif
 	d = (unsigned char *) SLrealloc ((char *) d, size);
@@ -961,7 +981,7 @@ unsigned char *remake_line(unsigned int size) /*{{{*/
 
 /*}}}*/
 
-void uniquely_name_buffer (Buffer *b, char *trry) /*{{{*/
+void uniquely_name_buffer (Buffer *b, SLFUTURE_CONST char *trry) /*{{{*/
 {
    Buffer *bnext;
    int version = 0, n;
@@ -1019,6 +1039,7 @@ Buffer *make_buffer (char *name, char *dir, char *file) /*{{{*/
    jed_set_buffer_ctime (newB);
    newB->local_vars.tab = Jed_Tab_Default;
    newB->local_vars.case_search = Jed_Case_Search_Default;
+   newB->local_vars.wrap_column = Jed_Wrap_Default;
 
    if (CBuf == NULL)
      {
@@ -1110,7 +1131,7 @@ void delete_buffer(Buffer *buf) /*{{{*/
    buf->prev->next = buf->next;
    buf->next->prev = buf->prev;
 
-#if JED_HAS_TTY_MENUS
+#if JED_HAS_MENUS
    jed_delete_menu_bar (buf->menubar);
 #endif
 
@@ -1353,7 +1374,7 @@ Buffer *find_file_buffer(char *file) /*{{{*/
 /*}}}*/
 
 /* take a dir and a filename, expand them then put in buffer structure */
-void buffer_filename(Buffer *b, char *dir, char *file) /*{{{*/
+void buffer_filename(Buffer *b, SLFUTURE_CONST char *dir, SLFUTURE_CONST char *file) /*{{{*/
 {
    unsigned int dirlen;
    char *dirfile;
@@ -1394,7 +1415,10 @@ void buffer_filename(Buffer *b, char *dir, char *file) /*{{{*/
 #endif
 
    if (*file)
-     uniquely_name_buffer (b, file);
+     {
+	uniquely_name_buffer (b, file);
+	jed_set_buffer_ctime (b);
+     }
 }
 
 /*}}}*/
@@ -1578,6 +1602,24 @@ int jed_set_buffer_hook (Buffer *buffer, char *name, SLang_Name_Type *nt)
    return 0;
 }
 
+SLang_Name_Type *jed_get_buffer_hook (Buffer *buffer, char *name)
+{
+   SLang_Name_Type **ntp;
+   Jed_Buffer_Hook_Type *b;
+
+   if (buffer == NULL)
+     return NULL;
+
+   b = buffer->buffer_hooks;
+
+   if ((b == NULL)
+       || (NULL == (ntp = find_buffer_hook (b, name)))
+       || (*ntp == NULL))
+     return NULL;
+
+   return SLang_copy_function (*ntp);
+}
+
 static void delete_buffer_hooks (Jed_Buffer_Hook_Type *h)
 {
    if (h == NULL)
