@@ -50,7 +50,7 @@ static Color_Object_Map_Type Color_Name_Map [JMAX_COLORS] =
 #if JED_HAS_LINE_ATTRIBUTES
    {"...",		JDOTS_COLOR, NULL, NULL},
 #endif   
-#if JED_HAS_TTY_MENUS
+#if JED_HAS_MENUS
    {"menu_char",	JMENU_CHAR_COLOR, NULL, NULL},
    {"menu_shadow",	JMENU_SHADOW_COLOR, NULL, NULL},
    {"menu_selection",	JMENU_SELECTION_COLOR, NULL, NULL},
@@ -66,17 +66,17 @@ static Color_Object_Map_Type Color_Name_Map [JMAX_COLORS] =
    {"italic",		JITALIC_COLOR, NULL, NULL},
    {"underline",	JUNDERLINE_COLOR, NULL, NULL},
    {"bold",		JBOLD_COLOR, NULL, NULL},
+   {"html",		JHTML_KEY_COLOR, NULL, NULL},
    {"keyword",		JKEY_COLOR, NULL, NULL},
-   {"keyword1",		JKEY_COLOR+1, NULL, NULL},
-   {"keyword2",		JKEY_COLOR+2, NULL, NULL},
-   {"keyword3",		JKEY_COLOR+3, NULL, NULL},
-   {"keyword4",		JKEY_COLOR+4, NULL, NULL},
-   {"keyword5",		JKEY_COLOR+5, NULL, NULL},
-   {"keyword6",		JKEY_COLOR+6, NULL, NULL},
-   {"keyword7",		JKEY_COLOR+7, NULL, NULL},
-   {"keyword8",		JKEY_COLOR+8, NULL, NULL},
-   {"keyword9",		JKEY_COLOR+9, NULL, NULL},
-#define FIRST_AVAILABLE_COLOR	(JKEY_COLOR+10)
+   {"keyword1",		JKEY1_COLOR, NULL, NULL},
+   {"keyword2",		JKEY2_COLOR, NULL, NULL},
+   {"keyword3",		JKEY3_COLOR, NULL, NULL},
+   {"keyword4",		JKEY4_COLOR, NULL, NULL},
+   {"keyword5",		JKEY5_COLOR, NULL, NULL},
+   {"keyword6",		JKEY6_COLOR, NULL, NULL},
+   {"keyword7",		JKEY7_COLOR, NULL, NULL},
+   {"keyword8",		JKEY8_COLOR, NULL, NULL},
+   {"keyword9",		JKEY9_COLOR, NULL, NULL},
    /* The rest of the colors are user-defined, and the strings are slstrings */
    /* %%% COLOR-TABLE-STOP %%% */
    {NULL, -1, NULL, NULL}
@@ -91,18 +91,27 @@ int jed_get_color_obj (char *name) /*{{{*/
    map = Color_Name_Map;
    map_max = Color_Name_Map + JMAX_COLORS;
    ch = *name;
-   while ((map < map_max) && (map->name != NULL))
+   while (map < map_max)
      {
+	if (map->name == NULL)
+	  {
+	     if (map < Color_Name_Map + FIRST_USER_COLOR)
+	       {
+		  map = Color_Name_Map + FIRST_USER_COLOR;
+		  continue;
+	       }
+	     break;
+	  }
 	if ((ch == map->name[0])
 	    && (0 == strcmp (map->name, name)))
 	  return map->color;
 
 	map++;
      }
-   
+
    if (0 == strcmp (name, "keyword0"))
      return JKEY_COLOR;
-
+	
    return -1;
 }
 
@@ -117,7 +126,7 @@ static int add_color_object (char *name)
    if (obj != -1)
      return obj;
    
-   map = Color_Name_Map + FIRST_AVAILABLE_COLOR;
+   map = Color_Name_Map + FIRST_USER_COLOR;
    map_max = Color_Name_Map + JMAX_COLORS;
    while (map < map_max)
      {
@@ -186,8 +195,8 @@ static void get_color (char *obj)
      i = JNORMAL_COLOR;		       
 
    map = Color_Name_Map + i;
-   (void) SLang_push_string (map->fg);
-   (void) SLang_push_string (map->bg);
+   (void) SLang_push_string (map->fg == NULL ? "default" : map->fg);
+   (void) SLang_push_string (map->bg == NULL ? "default" : map->bg);
 }
 
 static void add_color_object_cmd (char *name)
