@@ -6,12 +6,13 @@
  */
     
 /* For undo, these flags assumed to be <= 0xFF */
-#define CDELETE  0x1
-#define CINSERT  0x2
-#define LDELETE  0x4
-#define NLINSERT  0x8   /* causes marks to be moved to next line */
-#define NLDELETE  0x10    /* opposite of above */
-  
+#define CDELETE		0x1
+#define CINSERT		0x2
+#define LDELETE		0x4
+#define NLINSERT	0x8   /* causes marks to be moved to next line */
+#define NLDELETE	0x10    /* opposite of above */
+#define UNDO_POSITION	0x20	       /* records position */
+
 extern void jed_update_marks(int, int);  
 extern int No_Screen_Update;
 extern int jed_del_nbytes (int);
@@ -27,7 +28,7 @@ extern int jed_insert_nbytes(unsigned char *, int);
 extern int jed_insert_wchar_n_times (SLwchar_Type, unsigned int);
 extern int jed_del_newline(void);
 extern int _jed_replace_wchar (SLwchar_Type);   /* \n not allowed */
-extern int jed_insert_string (char *s);
+extern int jed_insert_string (SLFUTURE_CONST char *s);
 extern int jed_insert_newline (void);
 extern int Suspend_Screen_Update;
 extern int jed_prepare_for_modification (int);
