@@ -79,7 +79,6 @@
 
 Buffer *MiniBuffer;
 Buffer *The_MiniBuffer;     /* this never gets deleted since we may need it */
-Window_Type *The_MiniWindow;
 
 static Buffer *Last_Buffer;
 
@@ -87,8 +86,6 @@ static Buffer *Last_Buffer;
 int Ignore_Beep = 0;
 int MiniBuffer_Active = 0;
 
-char *Jed_Library;
-
 /* Do not change this value without changing file_findfirst/next. */
 #define JED_MAX_COMPLETION_LEN JED_MAX_PATH_LEN   
 
@@ -304,8 +301,8 @@ int kill_buffer_cmd(char *name) /*{{{*/
 
 int jed_save_buffer_to_file (char *dir, char *file)
 {
-   int status;
-   char *dirfile;
+   int status, state;
+   char *dirfile, *canonical_file;
 
    if ((file == NULL) || (*file == 0))
      {
@@ -320,15 +317,69 @@ int jed_save_buffer_to_file (char *dir, char *file)
    if (NULL == (dirfile = jed_dir_file_merge (dir, file)))
      return -1;
 
-   if (file_changed_on_disk (CBuf, dirfile) > 0)
+   state = 0;
+
+   canonical_file = jed_get_canonical_pathname (dirfile);
+   if (canonical_file == NULL)
      {
-	if (0 == jed_get_yes_no("File changed on disk.  Save anyway?"))
+	SLfree (dirfile);
+	return -1;
+     }
+
+   if (file_changed_on_disk (CBuf, dirfile) > 0)
+     state |= BUFFER_MODIFIED;
+   if (jed_file_is_readonly (canonical_file, 0))
+     state |= READ_ONLY;
+   if (0 != strcmp (canonical_file, CBuf->dirfile))
+     state |= OVERWRITE_MODE;
+
+   if (state)
+     {
+	char *prompt = NULL;
+
+	switch (state)
 	  {
+	   default:
+	   case OVERWRITE_MODE:
+	     prompt = NULL;
+	     break;
+
+	   case BUFFER_MODIFIED:
+	     prompt = "File changed on disk.  Save anyway?";
+	     break;
+	     
+	   case READ_ONLY:
+	     prompt = "No permission to write to file.  Try to save anyway?";
+	     break;
+	     
+	   case BUFFER_MODIFIED|READ_ONLY:
+	     prompt = "File changed on disk and no permission to write.  Save anyway?";
+	     break;
+	     
+	   case BUFFER_MODIFIED|OVERWRITE_MODE:
+	     prompt = "File exists and changed on disk.  Overwrite?";
+	     break;
+	     
+	   case READ_ONLY|OVERWRITE_MODE:
+	     prompt = "File exists and no permission to write.  Try to overwrite anyway?";
+	     break;
+	     
+	   case READ_ONLY|OVERWRITE_MODE|BUFFER_MODIFIED:
+	     prompt = "File changed on disk and no permission to write.  Overwrite anyway?";
+	     break;
+	  }
+	
+	if ((prompt != NULL) 
+	    && (1 != jed_get_yes_no(prompt)))
+	  {
+	     SLfree (canonical_file);
 	     SLfree (dirfile);
 	     return -1;
 	  }
      }
 
+   SLfree (canonical_file);
+
    if (-1 == jed_va_run_hooks ("_jed_save_buffer_before_hooks",
 			       JED_HOOKS_RUN_ALL, 1, dirfile))
      {
@@ -809,9 +860,9 @@ static void create_minibuffer(void) /*{{{*/
    
    w = JWindow;
    do other_window(); while (JWindow->next != w);
-   JWindow->next = The_MiniWindow;
-   The_MiniWindow->next = w;
-   The_MiniWindow->hscroll_column = 1;
+   JWindow->next = JMiniWindow;
+   JMiniWindow->next = w;
+   JMiniWindow->hscroll_column = 1;
    Mini_Info.action_window = w;
    other_window();    /* now in minibuffer window */
    window_buffer(MiniBuffer);
@@ -819,7 +870,7 @@ static void create_minibuffer(void) /*{{{*/
    MiniBuffer_Active = 1;
    erase_buffer ();
 
-#if JED_HAS_TTY_MENUS
+#if JED_HAS_MENUS
    jed_notify_menu_buffer_changed ();
 #endif
 
@@ -838,7 +889,7 @@ static int Last_Completion_Windows;
 /* evaluate command in minibuffer and leave */
 int exit_minibuffer() /*{{{*/
 {
-   if (IS_MINIBUFFER)
+   if (IN_MINI_WINDOW)
      {
 	if (Last_Completion_Buffer != NULL)
 	  {
@@ -859,7 +910,7 @@ int exit_minibuffer() /*{{{*/
 /*}}}*/
 
 /* return 1 if minibuffer already exists otherwise returns 0 */
-int select_minibuffer() /*{{{*/
+int select_minibuffer (void) /*{{{*/
 {
    Window_Type *w;
    
@@ -867,7 +918,7 @@ int select_minibuffer() /*{{{*/
    w = JWindow;
    while (MiniBuffer != NULL)
      {
-	if (JWindow->sy+1 == Jed_Num_Screen_Rows) return(1);
+	if (IN_MINI_WINDOW) return(1);
 	other_window();
 	if (w == JWindow) exit_error("Internal Error:  no window!", 1);
      }
@@ -1221,7 +1272,7 @@ char *pop_to_buffer(char *name) /*{{{*/
    /* find a window to use */
    do
      {
-	if (w->sy + 1 != Jed_Num_Screen_Rows)
+	if (0 == (w->flags & MINIBUFFER_WINDOW))
 	  {
 	     if (action != NULL)
 	       {
@@ -1371,15 +1422,18 @@ int pipe_region(char *cmd) /*{{{*/
 #endif
 
 
-static VFILE *jed_open_lib_file (char *file, char **dirfile) /*{{{*/
+static VFILE *jed_open_lib_file (SLFUTURE_CONST char *file, char **dirfile) /*{{{*/
 {
    char libfsl[JED_MAX_PATH_LEN], libfslc[JED_MAX_PATH_LEN];
-   char *lib, *type, *libf;
+   SLFUTURE_CONST char *lib, *type, *libf;
    unsigned int n;
    VFILE *vp = NULL;
+   int free_lib;
+   int delimiter;
 
    libf = file;
-   lib = Jed_Library;
+   lib = NULL;
+   free_lib = 0;
 
    /* If file begins with ./,then read it from the current directory */
    if ((0 == strncmp (file, "./", 2))
@@ -1394,16 +1448,22 @@ static VFILE *jed_open_lib_file (char *file, char **dirfile) /*{{{*/
      }
    else if (SLpath_is_absolute_path (file))
      lib = "";
-   else if ((lib == NULL) || (*lib == 0))
-     exit_error("JED_ROOT environment variable needs set.", 0);
+   else
+     {
+	lib = SLpath_get_load_path ();      /* SLstring */
+	free_lib = 1;
+	if ((lib == NULL) || (*lib == 0))
+	  exit_error("The JED_ROOT environment variable needs set.", 0);
+     }
 
    if ((NULL != (type = file_type(file)))
        && ((*type == 0) 
 	   || ((0 != strcmp (type, "sl")) && (0 != strcmp (type, "slc")))))
      type = NULL;
 
+   delimiter = SLpath_get_delimiter ();
    n = 0;
-   while (0 == SLextract_list_element (lib, n, ',', libfsl, sizeof(libfsl)))
+   while (0 == SLextract_list_element (lib, n, delimiter, libfsl, sizeof(libfsl)))
      {
 	n++;
 
@@ -1443,6 +1503,15 @@ static VFILE *jed_open_lib_file (char *file, char **dirfile) /*{{{*/
 	  }
      }
    
+   if (free_lib)
+     SLang_free_slstring ((char *) lib);
+
+   if (vp == NULL)
+     {
+	*dirfile = NULL;
+	return NULL;
+     }
+
    if (NULL == (*dirfile = SLang_create_slstring (libf)))
      {
 	vclose (vp);
@@ -1472,20 +1541,21 @@ static char *jed_read_from_file(SLang_Load_Type *x) /*{{{*/
 
 /*}}}*/
 
-int jed_ns_load_file (char *file, char *ns)
+int jed_ns_load_file (SLFUTURE_CONST char *file, SLFUTURE_CONST char *ns)
 {
    VFILE *vp;
    SLang_Load_Type *x;
    int ret;
+   char *dirfile;
 
-   if (NULL == (vp = jed_open_lib_file (file, &file)))
+   if (NULL == (vp = jed_open_lib_file (file, &dirfile)))
      {
-	SLang_verror (SL_OBJ_NOPEN, "Unable to load %s", file);
+	SLang_verror (SL_OBJ_NOPEN, "Unable to open %s.  Check the value of the S-Lang load path.", file);
 	return -1;
      }
 
 #if SLANG_VERSION >= 10409
-   x = SLns_allocate_load_type (file, ns);
+   x = SLns_allocate_load_type (dirfile, ns);
 #else
    if ((ns != NULL) 
        && (0 != strcmp (ns, "Global")))
@@ -1493,9 +1563,9 @@ int jed_ns_load_file (char *file, char *ns)
 	SLang_verror (SL_NOT_IMPLEMENTED, "loading a file into a namespace is not supported");
 	x = NULL;
      }
-   else x = SLallocate_load_type (file);
+   else x = SLallocate_load_type (dirfile);
 #endif
-   SLang_free_slstring (file);
+   SLang_free_slstring (dirfile);
    
    if (x == NULL)
      {
@@ -1556,7 +1626,7 @@ static char *jed_read_from_buffer (SLang_Load_Type *x) /*{{{*/
 }
 /*}}}*/
 
-void load_buffer (void) /*{{{*/
+void jed_load_buffer (char *ns) /*{{{*/
 {
    SLang_Load_Type *x;
    Buffer_Client_Type client_data;
@@ -1574,7 +1644,7 @@ void load_buffer (void) /*{{{*/
    if (file == NULL)
      return;
 	
-   x = SLallocate_load_type (file);
+   x = SLns_allocate_load_type (file, ns);
    SLfree (file);
    
    if (x == NULL)
@@ -1746,8 +1816,16 @@ int mini_complete (void) /*{{{*/
 	     
 	     if (last_key_char == '\t')
 	       {
-		  while (*pb) pb++;
-		  (void) jed_quick_insert ((unsigned char *)buf, (int) (pb - buf));
+		  if (complete_open == file_findfirst)
+		    {
+		       char *file = jed_extract_file_or_dir (buf);
+		       (void) jed_quick_insert ((unsigned char *) file, strlen(file));
+		    }
+		  else
+		    {
+		       while (*pb) pb++;
+		       (void) jed_quick_insert ((unsigned char *)buf, (int) (pb - buf));
+		    }
 		  newline ();
 	       }
 	  }
@@ -1781,7 +1859,7 @@ int mini_complete (void) /*{{{*/
 	bob ();
      }
    
-   while ((CBuf != MiniBuffer) || !IS_MINIBUFFER) other_window ();
+   while ((CBuf != MiniBuffer) || !IN_MINI_WINDOW) other_window ();
    
    if (n)
      {
@@ -1974,8 +2052,7 @@ void make_buffer_list(void) /*{{{*/
 
 /*}}}*/
 
-
-static void jed_traceback(char *s) /*{{{*/
+static void jed_traceback (SLFUTURE_CONST char *s) /*{{{*/
 {
    char *n;
 
@@ -2084,7 +2161,7 @@ static int run_init_slang_hooks (void)
    return 0;
 }
 
-static void slang_exit_error_handler (char *fmt, va_list ap)
+static void slang_exit_error_handler (SLFUTURE_CONST char *fmt, va_list ap)
 {
    char buf [2048];
    
@@ -2092,7 +2169,7 @@ static void slang_exit_error_handler (char *fmt, va_list ap)
    exit_error (buf, 0);
 }
 
-static void vmsg_hook (char *fmt, va_list ap)
+static void vmsg_hook (SLFUTURE_CONST char *fmt, va_list ap)
 {
    char buf [2048];
    SLvsnprintf (buf, sizeof (buf), fmt, ap);
@@ -2156,8 +2233,8 @@ void init_minibuffer() /*{{{*/
    /* do some initializing */
    switch_to_buffer(The_MiniBuffer);
    remake_line(132);
-   The_MiniWindow = create_window(Jed_Num_Screen_Rows-1, 0, 1, 1, Jed_Num_Screen_Cols);
-   The_MiniWindow->buffer = CBuf;
+   JMiniWindow = jed_create_minibuffer_window ();
+   JMiniWindow->buffer = CBuf;
    Buffer_Local.tab = 0;
    switch_to_buffer(tmp);
    SLang_Dump_Routine = jed_traceback;
@@ -2178,10 +2255,11 @@ void init_minibuffer() /*{{{*/
    if ((-1 == SLang_init_slang ())
 #ifndef SIXTEEN_BIT_SYSTEM
        || (-1 == SLang_init_slmath ())
+       || (-1 == SLang_init_array ())
 #endif
        || (-1 == SLang_init_posix_process ())
        || (-1 == SLang_init_posix_dir ())
-       || (-1 == SLang_init_slassoc ())
+       /* || (-1 == SLang_init_slassoc ()) -- handled by SLang_init_slang */
        || (-1 == SLang_init_stdio ())
        || (-1 == SLang_init_posix_io ())
        || (-1 == SLang_init_ospath ())
@@ -2190,7 +2268,7 @@ void init_minibuffer() /*{{{*/
 #endif
        || (-1 == init_jed_intrinsics ())
        || (-1 == jed_init_userinfo ())
-#if JED_HAS_TTY_MENUS
+#if JED_HAS_MENUS
        || (-1 == jed_init_menus ())
 #endif
        || (-1 == jed_init_syntax ())
@@ -2202,7 +2280,7 @@ void init_minibuffer() /*{{{*/
      }
    
    /* use jed rouotines instead of default slang ones */
-   SLang_Error_Hook = msg_error;
+   SLang_Error_Hook = jed_error_hook;
    SLang_VMessage_Hook = vmsg_hook;
    SLang_User_Clear_Error = jed_clear_error;
 #if SLANG_VERSION < 10409
