@@ -7,7 +7,7 @@
 
 extern Buffer *The_MiniBuffer;
 
-extern int jed_ns_load_file (char *, char *);
+extern int jed_ns_load_file (SLFUTURE_CONST char *, SLFUTURE_CONST char *);
 extern int find_file_in_window(char *);
 extern int find_file(void);
 extern int get_buffer(void);
@@ -35,7 +35,7 @@ extern int shell_command(char *);
 extern int pipe_region(char *);
 #endif
 
-extern void load_buffer(void);
+extern void jed_load_buffer(char *ns);
 extern void update_cmd(int *);
 extern void update_sans_update_hook_cmd (int *);
 extern void copy_region_cmd(char *);
@@ -67,7 +67,6 @@ extern char *what_buffer(void);
 extern void set_tab(int *);
 extern char *command_line_argv(int *);
 extern int MiniBuffer_Active;
-extern char *Jed_Library;
 extern FILE *jed_open_slang_file(char *, char *);
 extern char *Completion_Buffer;
 extern void set_expansion_hook (char *);
