@@ -68,7 +68,7 @@ int Screen_Row = 1;
 int Screen_Col = 1;
 int Cursor_Motion;    /* indicates cursor movement only -1 ^ v +1 < > */
 int Jed_Dollar = '$';
-int Jed_Highlight_Trailing_WS = 0;
+int Jed_Highlight_WS = 0;
 int User_Prefers_Line_Numbers = 0;
 int Mode_Has_Syntax_Highlight;
 int Wants_Syntax_Highlight = 1;	       /* if non-zero, highlight the syntax.
@@ -157,11 +157,18 @@ static void display_line (Line *line, int sy, int sx)
 
    num_columns = JWindow->width;
 
-   if (hscroll_col || sx || (CBuf->line_num_display_size))
+   if (hscroll_col || sx
+#if JED_HAS_DISPLAY_LINE_NUMBERS
+       || (CBuf->line_num_display_size)
+#endif
+      )
      {
-	int tmp = hscroll_col - (sx + CBuf->line_num_display_size);
-	SLsmg_set_screen_start (NULL, &tmp);
+	int tmp = hscroll_col - sx;
+#if JED_HAS_DISPLAY_LINE_NUMBERS
+	tmp -= CBuf->line_num_display_size;
 	num_columns -= CBuf->line_num_display_size;
+#endif
+	SLsmg_set_screen_start (NULL, &tmp);
 	sx = 0;
      }
 
@@ -215,7 +222,8 @@ static void display_line (Line *line, int sy, int sx)
 	  write_syntax_highlight (sy, line, len);
 	else
 	  {
-	     if (Jed_Highlight_Trailing_WS)
+	     if ((is_mini == 0) 
+		 && Jed_Highlight_WS & HIGHLIGHT_WS_TRAILING)
 	       {
 		  unsigned char *pmin = line->data;
 		  unsigned char *pmax = pmin + len;
@@ -234,6 +242,7 @@ static void display_line (Line *line, int sy, int sx)
 		    {
 		       SLsmg_set_color (JTWS_COLOR);
 		       SLsmg_write_nchars ((char *)p, pmax - p);
+		       SLsmg_set_color (0);
 		    }
 	       }
 	     else SLsmg_write_nchars ((char *)line->data, len);
@@ -298,7 +307,7 @@ static void display_line_numbers (void)
 {
    unsigned int i, imin, imax, linenum;
    Line *line_start, *line;
-   char buf[32], format[32];
+   char buf[32];
    unsigned int c;
 
    imin = JWindow->sy;
@@ -328,7 +337,6 @@ static void display_line_numbers (void)
 	line = line->prev;
      }
 
-   sprintf (format, "%%%dd ", CBuf->line_num_display_size-1);
    SLsmg_set_color (JLINENUM_COLOR);
    c = JWindow->sx;
 
@@ -348,7 +356,7 @@ static void display_line_numbers (void)
 	  }
 
 	SLsmg_gotorc (i, c);
-	sprintf (buf, format, linenum);
+	sprintf (buf, "%*d ", CBuf->line_num_display_size-1, linenum);
 	SLsmg_write_string (buf);
 
 	line = line->next;
@@ -558,7 +566,7 @@ Line *find_top (void)
 static void init_smg_for_buffer (int *rowp, int *colp)
 {
    SLsmg_Tab_Width = Buffer_Local.tab;
-   if (IS_MINIBUFFER)
+   if (IN_MINI_WINDOW)
      SLsmg_Newline_Behavior = SLSMG_NEWLINE_PRINTABLE;
    else
      SLsmg_Newline_Behavior = 0;
@@ -576,7 +584,7 @@ void point_column (int n)
 
    
    /* Compensate for the prompt */
-   if (IS_MINIBUFFER)
+   if (IN_MINI_WINDOW)
      n -= Mini_Info.effective_prompt_len;
 
    p = CLine->data;
@@ -636,7 +644,7 @@ void point_column (int n)
 
    FIX_CHAR_WIDTH;
 
-   if (IS_MINIBUFFER) n -= Mini_Info.effective_prompt_len;
+   if (IN_MINI_WINDOW) n -= Mini_Info.effective_prompt_len;
 
    p = CLine->data;
    pmax = p + CLine->len;
@@ -702,7 +710,7 @@ int calculate_column (void)
    c = 1 + jed_compute_effective_length (CLine->data, CLine->data + Point);
    Absolute_Column = c;
 
-   if (IS_MINIBUFFER) c += Mini_Info.effective_prompt_len;
+   if (IN_MINI_WINDOW) c += Mini_Info.effective_prompt_len;
    Screen_Col = c;
    return (c);
 }
@@ -753,7 +761,10 @@ void point_cursor (int c)
    if (cline == HScroll_Line) c -= HScroll;
    if (c < 1) c = 1; else if (c > JWindow->width) c = JWindow->width;
 
-   c += JWindow->sx + CBuf->line_num_display_size;
+   c += JWindow->sx;
+#if JED_HAS_DISPLAY_LINE_NUMBERS
+   c += CBuf->line_num_display_size;
+#endif
 
    SLsmg_gotorc (r - 1, c - 1);
    Screen_Row = r;
@@ -831,15 +842,25 @@ static void finish_status(int col_flag)
 
    while (1)
      {
+	char *v0 = v;
 	while (1)
 	  {
-	     ch = *v++;
-	     if (ch == 0) return;
+	     ch = *v;
+	     if (ch == 0)
+	       {
+		  SLsmg_write_nchars (v0, (unsigned int) (v-v0));
+		  return;
+	       }
 	     if (ch == '%')
-	       break;
-	     SLsmg_write_nchars (&ch, 1);
+	       {
+		  SLsmg_write_nchars (v0, (unsigned int) (v-v0));
+		  break;
+	       }
+	     v++;
 	  }
 
+	/* At this point *v == '%' */
+	v++;
 	ch = *v++;
 
 	switch (ch)
@@ -965,7 +986,8 @@ static int update_status_line (int col_flag)
    char buf[32], *b;
    int num;
 
-   if (JWindow->sy + 1 == Jed_Num_Screen_Rows) return 0;   /* minibuffer ? */
+   if (IN_MINI_WINDOW)
+     return 0;
 
    SLsmg_gotorc (JWindow->rows + JWindow->sy, 0);
    SLsmg_set_color (JSTATUS_COLOR);
@@ -1262,7 +1284,7 @@ static void compute_line_display_size (void)
 static int update_1(Line *top, int force)
 {
    int i;
-   Window_Type *w;
+   Window_Type *start_win;
    int did_eob, time_has_expired = 0;
 
    if (Batch ||
@@ -1322,7 +1344,7 @@ static int update_1(Line *top, int force)
 	return 1;
      }
 
-   w = JWindow;
+   start_win = JWindow;
    do
      {
 	int imax;
@@ -1353,7 +1375,7 @@ static int update_1(Line *top, int force)
 	  top = NULL;
 	else
 #endif
-	  mark_window_attributes ((w == JWindow) || (w->buffer != CBuf));
+	  mark_window_attributes ((start_win == JWindow) || (start_win->buffer != CBuf));
 
 	did_eob = 0;
 
@@ -1406,7 +1428,7 @@ static int update_1(Line *top, int force)
 	Mode_Has_Syntax_Highlight = 0;
 	if (!force && screen_input_pending (0))
 	  {
-	     while(JWindow != w) other_window();
+	     while (JWindow != start_win) other_window();
 	     JWindow->trashed = 1;  /* since cursor not pointed */
 #if SLANG_VERSION >= 20000
 	     /* (void) SLsmg_utf8_enable (1); */  /* default state */
@@ -1414,7 +1436,7 @@ static int update_1(Line *top, int force)
 
 	     return(0);
 	  }
-	else update_status_line(w != JWindow);
+	else update_status_line (start_win != JWindow);
 
 #if JED_HAS_DISPLAY_LINE_NUMBERS
 	if (CBuf->line_num_display_size)
@@ -1427,7 +1449,7 @@ static int update_1(Line *top, int force)
 	/* if (!JWindow->trashed) top = JWindow->beg.line; else  top = NULL; */
 
      }
-   while (JWindow != w);
+   while (JWindow != start_win);
 #if SLANG_VERSION >= 20000
    /* SLsmg_utf8_enable (1); */	       /* default state */
 #endif
@@ -1532,7 +1554,7 @@ static void update_minibuffer(void)
    if (MiniBuffer != NULL)
      {
 	w = JWindow;
-	while (!IS_MINIBUFFER) other_window();
+	while (!IN_MINI_WINDOW) other_window();
 
 	JWindow->beg.line = CLine;
 	mark_window_attributes (1);
@@ -1552,13 +1574,17 @@ static void update_minibuffer(void)
      display_line(NULL, Jed_Num_Screen_Rows-1, 0);
 }
 
+#if 0
 static void set_hscroll(int col)
 {
    int hdiff, whs = abs(Wants_HScroll), wc = JWindow->hscroll_column - 1;
-   int sw = Jed_Num_Screen_Cols - CBuf->line_num_display_size - 1;
+   int sw = Jed_Num_Screen_Cols - 1;
    static Line *last;
    Line *tmp;
 
+#if JED_HAS_DISPLAY_LINE_NUMBERS
+   sw -= CBuf->line_num_display_size;
+#endif
    if (sw < 1)
      sw = 1;
 
@@ -1623,6 +1649,94 @@ static void set_hscroll(int col)
 	  }
      }
 }
+#endif
+/* Let "|" denote the window edges, 
+ *   "." denote text, "*" denotes the current location
+ * 
+ *  .........|....*..........|....
+ *  <---- col --->
+ *  <-- wc -><-------sw------>
+ * The contraints are: wc >= 0, col >= 0, sw > 0.
+ * Let whs be the value of abs(Wants_HScroll).  We want to wc to satisfy
+ *   wc+1 <= col <= wc + sw - 1
+ * ==> col-sw+1 <= wc <= col-1
+ * However, if wc does not satisfy this, then set wc such that:
+ *
+ *   wc + whs <= col <= wc + sw - whs
+ *   col+whs-sw <= wc <= col-whs
+ * Evidently, this requires:
+ *    col+whs-sw < col-whs
+ *    whs-sw < -whs
+ *    2whs < sw ==> whs < sw/2
+ * 
+ * As a special case, if col < sw, then set wc=0.
+ */
+static void set_hscroll(int col)
+{
+   int whs = abs(Wants_HScroll), wc = JWindow->hscroll_column - 1;
+   int sw = Jed_Num_Screen_Cols-1;
+   static Line *last;
+   Line *tmp;
+   int wc_min, wc_max;
+
+#if JED_HAS_DISPLAY_LINE_NUMBERS
+   sw -= CBuf->line_num_display_size;
+#endif
+   if (sw < 2)
+     sw = 2;
+
+   if (Wants_HScroll > 0)
+     wc += HScroll;		       /* only this line scrolled */
+
+   /* take care of last effect of horizontal scroll */
+   if (last != NULL)
+     {
+	tmp = CLine;
+	CLine = last;
+	register_change(0);
+	CLine = tmp;
+	if (last != CLine)
+	  HScroll = 0;
+
+	last = NULL;
+     }
+   col--;			       /* use 0 origin */
+   if (2*whs > sw)
+     whs = sw/2;
+     
+   wc_min = col - sw + 1;
+   if (wc_min < 0) wc_min = 0;
+   wc_max = col - 1;
+   if (wc < wc_min)
+     wc = wc_min + whs;
+   if (wc > wc_max)
+     wc = wc_max - whs;
+
+   if (col < sw)
+     {
+	if ((CBuf->modes & WRAP_MODE)
+	    || (col <= sw - whs))
+	  wc = 0;
+     }
+
+   if (Wants_HScroll < 0)
+     {
+	/* Scroll whole window */
+	if (wc + 1 != JWindow->hscroll_column)
+	  {
+	     JWindow->hscroll_column = wc + 1;
+	     touch_window();
+	  }
+	HScroll = 0;
+     }
+   else
+     {
+	/* Scroll just this line -- do not change hscroll_column */
+	register_change(0);
+	last = HScroll_Line = CLine;
+	HScroll = wc - (JWindow->hscroll_column-1);
+     }
+}
 
 static char Top_Screen_Line_Buffer[132] = "If you see this, you have an installation problem.";
 
@@ -1637,7 +1751,7 @@ void define_top_screen_line (char *neew)
 
 static void update_top_screen_line (void)
 {
-#if JED_HAS_TTY_MENUS
+#if JED_HAS_MENUS
    if (Jed_Menus_Active
        || (Top_Window_SY > 0))
      {
@@ -1765,7 +1879,7 @@ void update(Line *line, int force, int flag, int run_update_hook)
      }
    if (!flag) *Error_Buffer = *Message_Buffer = 0;
 
-#if JED_HAS_TTY_MENUS
+#if JED_HAS_MENUS
    update_top_screen_line ();
 #else
    if ((Top_Window_SY > 0) && JScreen[0].is_modified)
