@@ -26,6 +26,31 @@ static unsigned char *write_using_color (unsigned char *p,
 					 unsigned char *pmax,
 					 int color)
 {
+   if ((color == 0) && (Jed_Highlight_WS & HIGHLIGHT_WS_TAB))
+     {
+	unsigned char *p1 = p;
+	while (p1 < pmax)
+	  {
+	     if (*p1 != '\t')
+	       {
+		  p1++;
+		  continue;
+	       }
+	     
+	     if (p1 != p)
+	       {
+		  SLsmg_set_color (0);
+		  SLsmg_write_nchars ((char *)p, (unsigned int)(p1-p));
+		  p = p1;
+	       }
+	     while ((p1 < pmax) && (*p1 == '\t'))
+	       p1++;
+	     SLsmg_set_color (JTAB_COLOR);
+	     SLsmg_write_nchars ((char *)p, (unsigned int)(p1-p));
+	     p = p1;
+	  }
+     }
+   /* drop */
    SLsmg_set_color (color);
 #if SLANG_VERSION < 20000
    SLsmg_write_nchars ((char *)p, (unsigned int) (pmax - p));
@@ -273,7 +298,7 @@ static unsigned char *write_whitespace (unsigned char *p, unsigned char *pmax, i
    if (p1 == p)
      return p;
 
-   if ((p1 == pmax) && Jed_Highlight_Trailing_WS)
+   if ((p1 == pmax) && (Jed_Highlight_WS & HIGHLIGHT_WS_TRAILING))
      return write_using_color (p, pmax, trailing_color);
 
    return write_using_color (p, p1, 0);
@@ -321,7 +346,7 @@ void write_syntax_highlight (int row, Line *l, unsigned int len)
    context = 0;
 #endif
 
-   if (context == JED_LINE_IN_COMMENT)
+   if (context & JED_LINE_IN_COMMENT)
      {
 	/* I do not like the whitespace that preceeds the
 	 * '*' on the current line to be highlighted.  So...
@@ -338,21 +363,21 @@ void write_syntax_highlight (int row, Line *l, unsigned int len)
 
 	p = highlight_comment (p, p, pmax, st);
      }
-   else if (context == JED_LINE_IN_STRING0)
+   else if (context & JED_LINE_IN_STRING0)
      {
 	p = highlight_string (p, pmax, st->quote_char, st->string_chars[0], 0);
      }
-   else if (context == JED_LINE_IN_STRING1)
+   else if (context & JED_LINE_IN_STRING1)
      {
 	p = highlight_string (p, pmax, st->quote_char, st->string_chars[1], 0);
      }
-   else if (context == JED_LINE_IN_HTML)
+   else if (context & JED_LINE_IN_HTML)
      {
 	p1 = p;
 	ch = st->sgml_stop_char;
 	while ((p1 < pmax) && (*p1++ != ch))
 	  ;
-	p = write_using_color (p, p1, JKEY_COLOR);
+	p = write_using_color (p, p1, JHTML_KEY_COLOR);
      }
    else if ((flags & FORTRAN_TYPE)
 	    && st->fortran_comment_chars[*p])
@@ -431,7 +456,7 @@ void write_syntax_highlight (int row, Line *l, unsigned int len)
 	     while ((p1 < pmax) && (Char_Syntax[*p1] == 0))
 	       p1++;
 	     if ((p1 != pmax)
-		 || (Jed_Highlight_Trailing_WS == 0))
+		 || ((Jed_Highlight_WS & HIGHLIGHT_WS_TRAILING) == 0))
 	       {
 		  p = write_using_color (p, p1, 0);
 		  continue;
@@ -487,14 +512,14 @@ void write_syntax_highlight (int row, Line *l, unsigned int len)
 		    }
 		  p1++;
 	       }
-	     p = write_using_color (p, p1, JKEY_COLOR);
+	     p = write_using_color (p, p1, JHTML_KEY_COLOR);
 	     continue;
 	  }
 
 	if (syntax & HTML_END_SYNTAX)  /* missed start from previous line */
 	  {
 	     /* FIXME!!! Start from beginning */
-	     p = write_using_color (p, p + 1, JKEY_COLOR);
+	     p = write_using_color (p, p + 1, JHTML_KEY_COLOR);
 	     continue;
 	  }
 
@@ -525,7 +550,8 @@ void write_syntax_highlight (int row, Line *l, unsigned int len)
 
 	if (syntax & QUOTE_SYNTAX)
 	  {
-	     p1 = p + 2;
+	     p1 = jed_multibyte_chars_forward (p, pmax, 2, NULL, 1);
+	     /* p1 = p + 2; */
 	     if (p1 < pmax)
 	       {
 		  p = write_using_color (p, p1, 0);
