@@ -101,7 +101,8 @@ static unsigned char *text_format_line(void) /*{{{*/
 
 /*}}}*/
 
-static int wrap_line1(int format) /*{{{*/
+/* returns 0 if to wrapped, 1 if wrapped, -1 if error */
+static int wrap_line1(int format, int trim) /*{{{*/
 {
    unsigned char *p, *pmin;
    int col;
@@ -112,18 +113,17 @@ static int wrap_line1(int format) /*{{{*/
      {
 	bol ();
 	/* Ignore leading whitespace */
-	/* pmin = jed_skip_whitespace (); */
-	pmin = CLine->data;
+	pmin = jed_skip_whitespace ();
+	/* pmin = CLine->data; */
      }
    if (pmin == NULL)
      return -1;
 
-   eol();
+   point_column(Buffer_Local.wrap_column - 1);
    col = calculate_column();
-   if (col < Jed_Wrap_Column)
-     return -1;
+   if ((col < Buffer_Local.wrap_column) && eolp ())
+     return 0;
 
-   point_column(Jed_Wrap_Column - 1);
    p = CLine->data + Point;
 
    while(p > pmin)
@@ -141,15 +141,16 @@ static int wrap_line1(int format) /*{{{*/
 	     if ((*pmin == ' ') || (*pmin == '\t')) break;
 	     pmin++;
 	  }
-	if (p == pmin) return -1;
+	if (p == pmin) return 0;
 	p = pmin;
      }
 
    jed_position_point (p);
-   (void)jed_trim_whitespace();
+   if (trim) 
+     (void)jed_trim_whitespace();
    (void) jed_insert_newline();
    jed_up(1);
-   return 0;
+   return 1;
 }
 
 /*}}}*/
@@ -158,7 +159,7 @@ int wrap_line (int format) /*{{{*/
 {
    int ret;
    push_spot();
-   ret = wrap_line1(format);
+   ret = wrap_line1(format, 1);
    pop_spot();
    return ret;
 }
@@ -305,7 +306,7 @@ static int mark_paragraph (Line **begp, Line **endp)
 int text_format_paragraph () /*{{{*/
 {
    unsigned char *p;
-   int n, col;
+   int indent_col, col;
    Line *end, *beg, *next;
    Jed_Buffer_Hook_Type *h = CBuf->buffer_hooks;
 
@@ -317,31 +318,58 @@ int text_format_paragraph () /*{{{*/
 
    push_spot();
    
-   get_current_indent(&n);
-   if (n + 1 >= Jed_Wrap_Column)
-     n = 0;
+   get_current_indent(&indent_col);
+   if (indent_col + 1 >= Buffer_Local.wrap_column)
+     indent_col = 0;
 
    if (-1 == mark_paragraph (&beg, &end))
      return 0;
 
    get_current_indent (&col);
-   if (col + 1 >= Jed_Wrap_Column)
-     indent_to (n);
+   /* col is the indentation of the first line of the paragraph-- don't change it */
+
+   bol ();
+   while (CLine != end)
+     {
+	if (-1 == wrap_line1 (0, 0))
+	  {
+	     pop_spot ();
+	     return -1;
+	  }
+	if (0 == jed_down (1))
+	  break;
+     }
+
+   while ((CLine != beg) 
+	  && (0 != jed_up (1)))
+     ;
+
+   if (col + 1 >= Buffer_Local.wrap_column)
+     indent_to (indent_col);
 
    bol ();
 
    /* Now loop formatting as we go until the end is reached */
    while(CLine != end)
      {
-	eol();
-	if (CLine != beg) indent_to(n);
-	if (-1 != wrap_line1(1))
+	int status;
+
+	/* eol(); */
+	if (CLine != beg) indent_to(indent_col);
+	status = wrap_line1(1, 1);
+	if (status == -1)
+	  {
+	     pop_spot ();
+	     return -1;
+	  }
+	if (status == 1)
 	  {
 	     (void) jed_down(1);
-	     indent_to(n);
+	     /* indent_to(indent_col); */
 	     continue;
 	  }
-	else if (CLine->next == end) break;
+	else if (CLine->next == end)
+	  break;
 
 	next = CLine->next;
 	if (next != end)
@@ -363,7 +391,7 @@ int text_format_paragraph () /*{{{*/
 	     col = calculate_column();
 
 	     /* FIXME for multibyte */
-	     if ((p - next->data) + col < Jed_Wrap_Column - 1)
+	     if ((p - next->data) + col < Buffer_Local.wrap_column - 1)
 	       {
 		  if (-1 == _jed_replace_wchar (' '))
 		    return -1;
@@ -392,11 +420,11 @@ int narrow_paragraph(void) /*{{{*/
    CHECK_READ_ONLY
    /* if (CBuf->modes != WRAP_MODE) return(0); */
    get_current_indent(&n);
-   wrap = Jed_Wrap_Column;
+   wrap = Buffer_Local.wrap_column;
    if (wrap - n <= wrap/2) return(0);
-   Jed_Wrap_Column -= n;
+   Buffer_Local.wrap_column -= n;
    text_format_paragraph();
-   Jed_Wrap_Column = wrap;
+   Buffer_Local.wrap_column = wrap;
    return(1);
 }
 
@@ -419,7 +447,7 @@ int center_line(void) /*{{{*/
 	p++;
      }
    if ((len = (int)(pmax - p)) < 0) len = 0;
-   if ((len = (Jed_Wrap_Column - len) / 2) < 0) len = 0;
+   if ((len = (Buffer_Local.wrap_column - len) / 2) < 0) len = 0;
    indent_to(len);
    pop_spot();
    return(1);
@@ -534,6 +562,12 @@ static void skip_chars_backward (char *range, int invert)
    int ignore_combining = 1;
    int no_newline;
 
+   if (*range == '^')
+     {
+	invert = !invert;
+	range++;
+     }
+
    if (NULL == (lut = SLwchar_strtolut ((SLuchar_Type *)range, 1, 1)))
      return;
 
@@ -574,6 +608,12 @@ static void skip_chars_forward (char *range, int invert)
    SLwchar_Lut_Type *lut;
    int ignore_combining = 1;
 
+   if (*range == '^')
+     {
+	invert = !invert;
+	range++;
+     }
+
    if (NULL == (lut = SLwchar_strtolut ((SLuchar_Type *)range, 1, 1)))
      return;
 
