@@ -194,6 +194,40 @@ static int unix_set_abort_char (unsigned char ch)
    return 0;
 }
 
+#define HAS_GETSETPGID (defined(HAVE_GETPGID) && defined(HAVE_SETPGID))
+#define HAS_TCGETSETPGRP (defined(HAVE_TCSETPGRP) && defined(HAVE_TCGETPGRP))
+
+#if HAS_TCGETSETPGRP
+static pid_t Terminal_PGID = -1;
+#endif
+#if HAS_GETSETPGID
+static pid_t Startup_PGID = -1;
+#endif
+
+static void set_process_group (void)
+{
+#if HAS_GETSETPGID && HAS_TCGETSETPGRP
+   if (-1 != (Terminal_PGID = tcgetpgrp (Read_FD)))
+     {
+	pid_t pid = getpid ();
+	Startup_PGID = getpgid (pid);
+	(void) tcsetpgrp (Read_FD, pid);
+	(void) setpgid (pid, pid);
+     }
+#endif
+}
+
+static void reset_process_group (void)
+{
+#if HAS_GETSETPGID && HAS_TCGETSETPGRP
+   if (Terminal_PGID != -1)
+     {
+	(void) tcsetpgrp (Read_FD, Terminal_PGID);
+	setpgid (0, Startup_PGID);
+     }
+#endif
+}
+
 int init_tty (void) /*{{{*/
 {
    TTY_Termio_Type newtty;
@@ -309,6 +343,9 @@ int init_tty (void) /*{{{*/
    X_Update_Open_Hook = unix_update_open;
    X_Update_Close_Hook = unix_update_close;
    X_Set_Abort_Char_Hook = unix_set_abort_char;
+   
+   set_process_group ();
+
    return 0;
 }
 
@@ -329,6 +366,8 @@ void reset_tty (void) /*{{{*/
 	if (errno != EINTR)
 	  break;
      }
+   
+   reset_process_group ();
 
    /* This statement ensures init_tty will not try to change output_rate
       (when coming back from suspension) */
@@ -344,9 +383,18 @@ void reset_tty (void) /*{{{*/
 
 /*}}}*/
 
+int Key_Wait_Time = 450;
+int jed_set_default_key_wait_time (const int *tsecs)
+{
+   int tmp = Key_Wait_Time;
+   if (*tsecs > 0)
+     Key_Wait_Time = *tsecs;
+   return tmp;
+}
+
 unsigned char sys_getkey (void) /*{{{*/
 {
-   int n = 450;
+   int n = Key_Wait_Time;
    unsigned char c;
 #if JED_HAS_SUBPROCESSES
    int all = Num_Subprocesses;
@@ -524,10 +572,15 @@ int sys_input_pending(int *tsecs, int all) /*{{{*/
 	i = 0;
 	while (i < Num_Subprocesses)
 	  {
-	     FD_SET(Subprocess_Read_fds[i][0], &Read_FD_Set);
+	     
+	     if (Subprocess_Read_fds[i][2] == 0)   /* If non-0, fd has an EIO error */
+	       {
+		  int fd = Subprocess_Read_fds[i][0];
+		  FD_SET(fd, &Read_FD_Set);
+		  if (fd > maxfd) maxfd = fd;
+	       }
 	     i++;
 	  }
-	if (Max_Subprocess_FD > maxfd) maxfd = Max_Subprocess_FD;
      }
 #endif
 
@@ -633,7 +686,8 @@ int sys_input_pending(int *tsecs, int all) /*{{{*/
    i = 0;
    while (i < Num_Subprocesses)
      {
-	if (FD_ISSET(Subprocess_Read_fds[i][0], &Read_FD_Set))
+	if ((Subprocess_Read_fds[i][2] == 0)   /* If non-0, fd has an EIO error */
+	    && FD_ISSET(Subprocess_Read_fds[i][0], &Read_FD_Set))
 	  read_process_input (Subprocess_Read_fds[i][1]);
 	i++;
      }
@@ -687,10 +741,10 @@ void sys_suspend (void) /*{{{*/
 /*}}}*/
 
 /* returns 0 if file does not exist, 1 if it is not a dir, 2 if it is */
-int sys_chmod(char *file, int what, mode_t *mode, uid_t *uid, gid_t *gid) /*{{{*/
+int sys_chmod(SLFUTURE_CONST char *file, int what, mode_t *mode, uid_t *uid, gid_t *gid) /*{{{*/
 {
    struct stat buf;
-   int m;
+   mode_t m;
 
    if (what)
      {
