@@ -28,6 +28,9 @@
 # include <X11/keysym.h>
 # include <X11/cursorfont.h>
 # include <X11/Intrinsic.h>
+#ifndef XJED_HAS_XRENDERFONT
+# define XJED_HAS_XRENDERFONT 0
+#endif
 # if XJED_HAS_XRENDERFONT
 #  include <X11/Xft/Xft.h>
 # endif
@@ -212,6 +215,8 @@ static char *Iconic = NULL;
 static Atom Xjed_Prop;
 static Atom Compound_Text_Atom;
 static Atom UTF8_String_Atom;
+static Atom Text_Atom;
+static Atom Targets_Atom;
 
 static XEvent Current_Event;
 static char *Selection_Send_Data = NULL;
@@ -1488,19 +1493,26 @@ static int X_read_key (void) /*{{{*/
 static int X_input_pending (void) /*{{{*/
 {
    XEvent ev;
+
    int n;
 
    if (No_XEvents) return 0;
-
+#if 1
+   /* XCheckMaskEvent searches the queue and removes the specified event. */
+   if (0 != XCheckMaskEvent(This_XDisplay, KeyPressMask, &ev))
+     {
+	/* XPutBackEvent pushes the event to the head of the event queue-- no return value */
+	XPutBackEvent(This_XDisplay, &ev);
+	return 1;
+     }
+#endif
    n = XPending (This_XDisplay);
-   if (!n) return (0);
-
-   /* I need some way of getting only kbd events. */
-   while (n--)
+   while (n > 0)
      {
 	XPeekEvent(This_XDisplay, &ev);
 	if (0 == x_handle_harmless_events (&ev)) return 1;
 	XNextEvent(This_XDisplay, &ev);
+	n--;
      }
    return 0;
 }
@@ -1609,6 +1621,23 @@ static void xjed_suspend (void) /*{{{*/
 
 /*}}}*/
 
+static void x_toggle_visibility (void) /*{{{*/
+{
+   int hide_win = (XWin->visible == VisibilityUnobscured);
+
+   if ((SLang_Num_Function_Args == 1)
+       && (-1 == SLang_pop_integer (&hide_win)))
+     return;
+
+   if (hide_win)
+     /* XIconifyWindow (This_XDisplay, XWin->w, This_XScreen); */
+     XLowerWindow (This_XDisplay, This_XWindow);
+   else
+     XRaiseWindow (This_XDisplay, This_XWindow);
+}
+/*}}}*/
+
+
 static int get_font_width (XFontStruct *f, int *wp, int *is_dualp)
 {
    int w0, w1;
@@ -1652,7 +1681,7 @@ static int load_font (char *font) /*{{{*/
 				    XFT_FAMILY, XftTypeString, font, 
 				    XFT_SIZE, XftTypeDouble, XWin->face_size,
 				    XFT_SPACING, XftTypeInteger, XFT_MONO,
-				    0);
+				    NULL);
 	if (XWin->xftfont == NULL) return -1;
 	XWin->font_name = font;
 	XWin->font_height = XWin->xftfont->ascent + XWin->xftfont->descent;
@@ -2271,7 +2300,9 @@ static int init_Xdisplay (void) /*{{{*/
    i18init ();
 #endif
    Compound_Text_Atom = XInternAtom(This_XDisplay, "COMPOUND_TEXT", False);
+   Text_Atom = XInternAtom(This_XDisplay, "TEXT", False);
    UTF8_String_Atom = XInternAtom (This_XDisplay, "UTF8_STRING", False);
+   Targets_Atom = XInternAtom (This_XDisplay, "TARGETS", False);
    Xjed_Prop = XInternAtom(This_XDisplay, "XJED_PROPERTY_TEXT", False);
    return ConnectionNumber (This_XDisplay);
 }
@@ -2880,59 +2911,99 @@ static int send_selection (XEvent *ev)
    int len;
    XTextProperty tp;
    XSelectionEvent sev;
+   XSelectionRequestEvent *xsr;
    int status;
    Atom target;
-   int (*text_to_property)(Display *, char **, int, XICCEncodingStyle, XTextProperty *);
-   XICCEncodingStyle style;
+   int free_tp_value;
 
    if (NULL == Selection_Send_Data) 
      return 0;
 
    memset ((char *)&tp, 0, sizeof (tp));
 
-   len = strlen (Selection_Send_Data);
-   target = ev->xselectionrequest.target;
+   xsr = &ev->xselectionrequest;
+   target = xsr->target;
 
-#if USE_XUTF8_CODE
-   if (Jed_UTF8_Mode)
-     text_to_property = Xutf8TextListToTextProperty;
-   else
-#endif
-     text_to_property = XmbTextListToTextProperty;
+   if (target == Targets_Atom)
+     {
+	/* The requester wants to know what targets we support.  How polite. */
+#define MAX_SELECTION_TARGETS 5
+	Atom target_atoms[MAX_SELECTION_TARGETS];
+	unsigned int ntargets = 0;
 
-   if (target == Compound_Text_Atom)
-     style = XCompoundTextStyle;
+	target_atoms[ntargets++] = XA_STRING;
+	target_atoms[ntargets++] = Text_Atom;
+	target_atoms[ntargets++] = Compound_Text_Atom;
 #if USE_XUTF8_CODE
-   else if (target == UTF8_String_Atom)
-     style = XUTF8StringStyle;
+	target_atoms[ntargets++] = UTF8_String_Atom;
 #endif
+	tp.value = (unsigned char *)target_atoms;
+	tp.format = 8;
+	tp.nitems = sizeof(Atom)*ntargets;
+	free_tp_value = 0;
+	len = 0;
+     }
    else
      {
-	style = XStringStyle;
-	target = XA_STRING;
-     }   
+	int (*text_to_property)(Display *, char **, int, XICCEncodingStyle, XTextProperty *);
+	XICCEncodingStyle style;
 
-   status = (*text_to_property) (This_XDisplay, &Selection_Send_Data, 1, style, &tp);
+#if USE_XUTF8_CODE
+	if (Jed_UTF8_Mode)
+	  text_to_property = Xutf8TextListToTextProperty;
+	else
+#endif
+	  text_to_property = XmbTextListToTextProperty;
 
-   if ((status != Success) || (tp.value == NULL))
-     return -1;
+	if (target == Compound_Text_Atom)
+	  style = XCompoundTextStyle;
+	else if (target == UTF8_String_Atom)
+	  {
+#if USE_XUTF8_CODE
+	     style = XUTF8StringStyle;
+#else
+	     style = XTextStyle;
+#endif
+	  }
+	else if ((target == Text_Atom) || (target == XA_STRING))
+	  style = XTextStyle;
+	else
+	  {
+	     char *name = XGetAtomName(This_XDisplay, target);
+	     if (name != NULL)
+	       {
+		  (void) fprintf (stderr, "Unsupported selection target: %s\n", name);
+		  XFree (name);
+	       }
+	     return -1;
+	  }   
+	status = (*text_to_property) (This_XDisplay, &Selection_Send_Data, 1, style, &tp);
+
+	if ((status != Success) || (tp.value == NULL))
+	  return -1;
+	
+	free_tp_value = 1;
+	len = strlen (Selection_Send_Data);
+     }
+
+   status = XChangeProperty (This_XDisplay, xsr->requestor,
+			     xsr->property, xsr->target, tp.format,
+			     PropModeReplace, tp.value, tp.nitems);
 
    sev.type = SelectionNotify;
-   sev.requestor = ev->xselectionrequest.requestor;
-   sev.selection = ev->xselectionrequest.selection;
+   sev.requestor = xsr->requestor;
+   sev.selection = xsr->selection;
    sev.target = target;
-   sev.time = ev->xselectionrequest.time;
-   sev.property = ev->xselectionrequest.property;
-   status = XChangeProperty (This_XDisplay, ev->xselectionrequest.requestor,
-			     ev->xselectionrequest.property, ev->xselectionrequest.target, tp.format,
-			     PropModeReplace, tp.value, tp.nitems);
+   sev.time = xsr->time;
+   sev.property = xsr->property;
    /* Apparantly XChangeProperty can return BadRequest, even if it succeeds.
     * So ignore its return value except for BadAlloc
     */
    if (status != BadAlloc)
-     (void) XSendEvent (This_XDisplay, ev->xselectionrequest.requestor, False, (long)NULL, (XEvent*)&sev);
+     (void) XSendEvent (This_XDisplay, xsr->requestor, False, (long)NULL, (XEvent*)&sev);
 
-   XFree (tp.value);
+   if (free_tp_value)
+     XFree (tp.value);
 
    return len;
 }
@@ -3011,6 +3082,7 @@ static SLang_Intrin_Fun_Type sl_x_table[] = /*{{{*/
 #if SLANG_VERSION < 10404
    MAKE_INTRINSIC_S("get_termcap_string", get_termcap_string, STRING_TYPE),
 #endif
+   MAKE_INTRINSIC_0("x_toggle_visibility", x_toggle_visibility, SLANG_VOID_TYPE),
    MAKE_INTRINSIC(NULL,NULL,0,0)
 };
 
@@ -3182,6 +3254,9 @@ static void JX_get_terminfo (void) /*{{{*/
 {
    SLsmg_Term_Type tt;
 
+#ifdef REAL_UNIX_SYSTEM
+   SLtt_Force_Keypad_Init = 1;
+#endif
    if ((Batch) || !open_Xdisplay())
     {
        /* This function should match the corresponding function in display.c.
